@@ -42,7 +42,7 @@
 #  include "gphoto2-cmd-capture.h"
 #endif
 
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 #  include <libexif/exif-data.h>
 #endif
 
@@ -83,7 +83,43 @@ action_camera_upload_file (GPParams *p, const char *folder, const char *path)
 	res = gp_camera_folder_put_file (p->camera, folder, file,
 					 p->context);
 	gp_file_unref (file);
+	return (res);
+}
 
+int
+action_camera_upload_metadata (GPParams *p, const char *folder, const char *path)
+{
+	CameraFile *file;
+	int res;
+
+	gp_log (GP_LOG_DEBUG, "main", "Uploading metadata...");
+
+	CR (gp_file_new (&file));
+	res = gp_file_open (file, path);
+	if (res < 0) {
+		gp_file_unref (file);
+		return (res);
+	}
+
+	/* Check if the user specified a filename */
+	if (p->filename && strcmp (p->filename, "")) {
+		res = gp_file_set_name (file, p->filename);
+		if (res < 0) {
+			gp_file_unref (file);
+			return (res);
+		}
+	} else if (path == strstr(path, "meta_")) {
+		res = gp_file_set_name (file, path+5);
+		if (res < 0) {
+			gp_file_unref (file);
+			return (res);
+		}
+	}
+	gp_file_set_type (file, GP_FILE_TYPE_METADATA);
+
+	res = gp_camera_folder_put_file (p->camera, folder, file,
+					 p->context);
+	gp_file_unref (file);
 	return (res);
 }
 
@@ -140,21 +176,11 @@ list_folders_action (GPParams *p)
 	CL (gp_camera_folder_list_folders (p->camera, p->folder, list,
 					   p->context), list);
 	CL (count = gp_list_count (list), list);
-	switch (count) {
-        case 0:
-                printf (_("There are no folders in folder '%s'."), p->folder);
-                putchar ('\n');
-                break;
-        case 1:
-                printf (_("There is one folder in folder '%s':"), p->folder);
-                putchar ('\n');
-                break;
-        default:
-                printf (_("There are %i folders in folder '%s':"),
-			 count, p->folder);
-                putchar ('\n');
-                break;
-	}
+        printf(ngettext(
+		"There is %d folder in folder '%s'.\n", 
+		"There are %d folders in folder '%s'.\n",
+		count
+	), count, p->folder);
 	for (i = 0; i < count; i++) {
 		CL (gp_list_get_name (list, i, &name), list);
 		printf (" - %s\n", name);
@@ -190,22 +216,15 @@ list_files_action (GPParams *p)
 	}
 	else
 	  filecount = count;
-
-	switch (filecount) {
-	case 0:
-		printf (_("There are no files in folder '%s'."), p->folder);
-                putchar ('\n');
-		break;
-	case 1:
-		printf (_("There is one file in folder '%s':"), p->folder);
-                putchar ('\n');
-		break;
-	default:
-		printf (_("There are %i files in folder '%s':"),
-			filecount, p->folder);
-                putchar ('\n');
-		break;
-	}
+        if (filecount == 0) { /* 0 is weird still, despite ngettext() */
+	   printf(_("There is no file in folder '%s'.\n"), p->folder);
+        } else {
+	   printf(ngettext(
+		"There is %d file in folder '%s'.\n",
+		"There are %d files in folder '%s'.\n",
+		filecount
+	   ), filecount, p->folder);
+        }
 	for (i = 0; i < count; i++) {
 		CL (gp_list_get_name (list, i, &name), list);
 		CL (print_file_action (p, name), list);
@@ -348,6 +367,13 @@ save_exif_action (GPParams *p, const char *filename)
 				   p->folder, filename, GP_FILE_TYPE_EXIF));
 }
 
+int
+save_meta_action (GPParams *p, const char *filename)
+{
+	return (save_file_to_file (p->camera, p->context, p->flags,
+				   p->folder, filename, GP_FILE_TYPE_METADATA));
+}
+
 int
 save_thumbnail_action (GPParams *p, const char *filename)
 {
@@ -397,7 +423,7 @@ delete_file_action (GPParams *p, const char *filename)
 				       p->context));
 }
 
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 static void
 show_ifd (ExifContent *content)
 {
@@ -408,7 +434,7 @@ show_ifd (ExifContent *content)
                 e = content->entries[i];
                 printf ("%-20.20s", exif_tag_get_name (e->tag));
                 printf ("|");
-#ifdef HAVE_EXIF_0_6_9
+#ifdef HAVE_LIBEXIF_LOG
 		{char b[1024];
 		printf ("%-59.59s", exif_entry_get_value (e, b, sizeof (b)));
 		}
@@ -436,12 +462,12 @@ print_hline (void)
 int
 print_exif_action (GPParams *p, const char *filename)
 {
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
         CameraFile *file;
         const char *data;
         unsigned long size;
         ExifData *ed;
-#ifdef HAVE_EXIF_0_5_4
+#ifdef HAVE_LIBEXIF_IFD
 	unsigned int i;
 #endif
 
@@ -464,7 +490,7 @@ print_exif_action (GPParams *p, const char *filename)
         printf ("%-59.59s", _("Value"));
         putchar ('\n');
         print_hline ();
-#ifdef HAVE_EXIF_0_5_4
+#ifdef HAVE_LIBEXIF_IFD
 	for (i = 0; i < EXIF_IFD_COUNT; i++)
 		if (ed->ifd[i])
 			show_ifd (ed->ifd[i]);
@@ -539,24 +565,38 @@ list_cameras_action (GPParams *p)
 	return (r);
 }
 
-int
-list_ports_action (GPParams *p)
-{
+static void
+_get_portinfo_list (GPParams *p) {
+	int count, result;
 	GPPortInfoList *list = NULL;
-	GPPortInfo info;
-	int x, count, result = GP_OK;
 
-	CR (gp_port_info_list_new (&list));
+	if (p->portinfo_list)
+		return;
+
+	if (gp_port_info_list_new (&list) < GP_OK)
+		return;
 	result = gp_port_info_list_load (list);
 	if (result < 0) {
 		gp_port_info_list_free (list);
-		return (result);
+		return;
 	}
 	count = gp_port_info_list_count (list);
 	if (count < 0) {
 		gp_port_info_list_free (list);
-		return (count);
+		return;
 	}
+	p->portinfo_list = list;
+	return;
+}
+
+int
+list_ports_action (GPParams *p)
+{
+	GPPortInfo info;
+	int x, count, result = GP_OK;
+
+	_get_portinfo_list (p);
+	count = gp_port_info_list_count (p->portinfo_list);
 
 	if (p->flags & FLAGS_QUIET)
 		printf("%i\n", count);
@@ -568,14 +608,11 @@ list_ports_action (GPParams *p)
 
 	/* Now list the ports */
 	for (x = 0; x < count; x++) {
-		result = gp_port_info_list_get_info (list, x, &info);
+		result = gp_port_info_list_get_info (p->portinfo_list, x, &info);
 		if (result < 0)
 			break;
 		printf ("%-32s %-32s\n", info.path, info.name);
 	}
-
-	gp_port_info_list_free (list);
-
 	return (result);
 
 }
@@ -586,17 +623,16 @@ auto_detect_action (GPParams *p)
 	int x, count;
         CameraList *list;
         CameraAbilitiesList *al = NULL;
-        GPPortInfoList *il = NULL;
         const char *name = NULL, *value = NULL;
 
+	_get_portinfo_list (p);
+	count = gp_port_info_list_count (p->portinfo_list);
+
 	CR (gp_list_new (&list));
         gp_abilities_list_new (&al);
         gp_abilities_list_load (al, p->context);
-        gp_port_info_list_new (&il);
-        gp_port_info_list_load (il);
-        gp_abilities_list_detect (al, il, list, p->context);
+        gp_abilities_list_detect (al, p->portinfo_list, list, p->context);
         gp_abilities_list_free (al);
-        gp_port_info_list_free (il);
 
         CL (count = gp_list_count (list), list);
 
@@ -652,9 +688,12 @@ action_camera_show_abilities (GPParams *p)
 	}
 	printf (_("Configuration support            : %s\n"),
 		(a.operations & GP_OPERATION_CONFIG) ? _("yes"):_("no"));
-	printf (_("Delete files on camera support   : %s\n"),
+	printf (_("Delete selected files on camera  : %s\n"),
 		(a.file_operations & GP_FILE_OPERATION_DELETE) ?
 							_("yes"):_("no"));
+        printf (_("Delete all files on camera       : %s\n"),
+        	(a.folder_operations & GP_FOLDER_OPERATION_DELETE_ALL) ?
+                					_("yes"):_("no"));
 	printf (_("File preview (thumbnail) support : %s\n"),
 		(a.file_operations & GP_FILE_OPERATION_PREVIEW) ? 
 							_("yes"):_("no"));
@@ -668,7 +707,6 @@ action_camera_show_abilities (GPParams *p)
 int
 action_camera_set_port (GPParams *params, const char *port)
 {
-	GPPortInfoList *il = NULL;
 	int p, r;
 	GPPortInfo info;
 	char verified_port[1024];
@@ -701,17 +739,12 @@ action_camera_set_port (GPParams *params, const char *port)
 		strncpy (verified_port, port, sizeof (verified_port) - 1);
 
 	/* Create the list of ports and load it. */
-	r = gp_port_info_list_new (&il);
-	if (r < 0)
-		return (r);
-	r = gp_port_info_list_load (il);
-	if (r < 0) {
-		gp_port_info_list_free (il);
-		return (r);
-	}
+	_get_portinfo_list (params);
 
 	/* Search our port in the list. */
-	p = gp_port_info_list_lookup_path (il, verified_port);
+	/* NOTE: This call can modify "il" for regexp matches! */
+	p = gp_port_info_list_lookup_path (params->portinfo_list, verified_port);
+
 	switch (p) {
 	case GP_ERROR_UNKNOWN_PORT:
 		fprintf (stderr, _("The port you specified "
@@ -725,14 +758,9 @@ action_camera_set_port (GPParams *params, const char *port)
 	default:
 		break;
 	}
-	if (p < 0) {
-		gp_port_info_list_free (il);
-		return (p);
-	}
 
 	/* Get info about our port. */
-	r = gp_port_info_list_get_info (il, p, &info);
-	gp_port_info_list_free (il);
+	r = gp_port_info_list_get_info (params->portinfo_list, p, &info);
 	if (r < 0)
 		return (r);
 
@@ -741,7 +769,6 @@ action_camera_set_port (GPParams *params, const char *port)
 	if (r < 0)
 		return (r);
 	gp_setting_set ("gphoto2", "port", info.path);
-
 	return (GP_OK);
 }
 
@@ -844,6 +871,12 @@ int
 print_version_action (GPParams *p)
 {
 	int n;
+	const char *port_message =
+#ifdef OS2
+			_("OS/2 port by Bart van Leeuwen\n");
+#else
+			"";
+#endif
 	printf (_("gphoto2 %s\n"
 		  "\n"
 		  "Copyright (c) 2000-2004 Lutz Mueller and others\n"
@@ -854,13 +887,8 @@ print_version_action (GPParams *p)
 		  "License. For more information about these matters, see the files named COPYING.\n"
 		  "\n"
 		  "This version of gphoto2 is using the following software versions and options:\n"),
-
 		VERSION,
-#ifdef OS2
-			_("OS/2 port by Bart van Leeuwen\n")
-#else
-			""
-#endif
+		port_message
 		);
 
 	for (n = 0; module_versions[n].name != NULL; n++) {
@@ -911,6 +939,40 @@ action_camera_capture_preview (GPParams *p)
 	return (GP_OK);
 }
 
+int
+action_camera_wait_event (GPParams *p)
+{
+	int ret;
+	CameraEventType	event;
+	void	*data = NULL;
+	CameraFilePath	*fn;
+
+	ret = gp_camera_wait_for_event (p->camera, 10000, &event, &data, p->context);
+	if (ret != GP_OK)
+		return ret;
+	switch (event) {
+	case GP_EVENT_UNKNOWN:
+		if (data) {
+			printf("UNKNOWN %s\n", (char*)data);
+		} else {
+			printf("UNKNOWN\n");
+		}
+		break;
+	case GP_EVENT_TIMEOUT:
+		printf("TIMEOUT\n");
+		break;
+	case GP_EVENT_FILE_ADDED:
+		fn = (CameraFilePath*)data;
+		printf("FILEADDED %s %s\n",fn->name, fn->folder);
+		break;
+	case GP_EVENT_FOLDER_ADDED:
+		fn = (CameraFilePath*)data;
+		printf("FOLDERADDED %s %s\n",fn->name, fn->folder);
+		break;
+	}
+	return GP_OK;
+}
+
 int
 override_usbids_action (GPParams *p, int usb_vendor, int usb_product,
 			int usb_vendor_modified, int usb_product_modified)
@@ -1103,6 +1165,23 @@ _find_widget_by_name (GPParams *p, const char *name, CameraWidget **child, Camer
 	return GP_OK;
 }
 
+/* From the strftime(3) man page:
+ * BUGS
+ *     Some buggy versions of gcc complain about the use of %c: warning:
+ *     %c yields only last 2 digits of year in some  locales.
+ *     Of course programmers are encouraged to use %c, it gives the
+ *     preferred date and time representation. One meets all kinds of
+ *     strange obfuscations to circumvent this gcc problem. A relatively
+ *     clean one is to add an intermediate function
+ */
+
+static inline size_t
+my_strftime(char *s, size_t max, const char *fmt, const struct tm *tm) {
+	return strftime(s, max, fmt, tm);
+}
+
+
+
 int
 get_config_action (GPParams *p, const char *name) {
 	CameraWidget *rootconfig,*child;
@@ -1181,7 +1260,7 @@ get_config_action (GPParams *p, const char *name) {
 		}
 		xtime = t;
 		xtm = localtime (&xtime);
-		ret = strftime (timebuf, sizeof(timebuf), "%c", xtm);
+		ret = my_strftime (timebuf, sizeof(timebuf), "%c", xtm);
 		printf ("Type: DATE\n");
 		printf ("Current: %d\n", t);
 		printf ("Printable: %s\n", timebuf);
