@@ -39,6 +39,7 @@
 #include <stdarg.h>
 #include <string.h>
 #include <unistd.h>
+#include <ctype.h>
 #include <locale.h>
 
 #ifdef HAVE_POPT
@@ -102,7 +103,7 @@ OPTION_CALLBACK(abilities);
 #ifdef HAVE_CDK
 OPTION_CALLBACK(config);
 #endif
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 OPTION_CALLBACK(show_exif);
 #endif
 OPTION_CALLBACK(show_info);
@@ -131,6 +132,7 @@ OPTION_CALLBACK(num_files);
 OPTION_CALLBACK(get_file);
 OPTION_CALLBACK(get_all_files);
 OPTION_CALLBACK(get_thumbnail);
+OPTION_CALLBACK(get_all_metadata);
 OPTION_CALLBACK(get_all_thumbnails);
 OPTION_CALLBACK(get_raw_data);
 OPTION_CALLBACK(get_all_raw_data);
@@ -139,6 +141,7 @@ OPTION_CALLBACK(get_all_audio_data);
 OPTION_CALLBACK(delete_file);
 OPTION_CALLBACK(delete_all_files);
 OPTION_CALLBACK(upload_file);
+OPTION_CALLBACK(upload_metadata);
 OPTION_CALLBACK(capture_frames);
 OPTION_CALLBACK(capture_interval);
 OPTION_CALLBACK(capture_image);
@@ -153,6 +156,7 @@ OPTION_CALLBACK(remove_dir);
 OPTION_CALLBACK(list_config);
 OPTION_CALLBACK(get_config);
 OPTION_CALLBACK(set_config);
+OPTION_CALLBACK(wait_event);
 
 /* 2) Add an entry in the option table                          */
 /*    ----------------------------------------------------------------- */
@@ -199,6 +203,9 @@ Option option[] = {
 {"P", "get-all-files","",        N_("Get all files from folder"),   get_all_files,0},
 {"t", "get-thumbnail",  "range",  N_("Get thumbnails given in range"),  get_thumbnail,  0},
 {"T", "get-all-thumbnails","",    N_("Get all thumbnails from folder"), get_all_thumbnails,0},
+{"", "get-metadata","",    N_("Get metadata from file"), get_metadata,0},
+{"", "get-all-metadata","",    N_("Get all metadata from folder"), get_all_metadata,0},
+{"", "upload-metadata","",    N_("Upload metadata for file"), upload_metadata,0},
 {"", "get-raw-data", "range",     N_("Get raw data given in range"),    get_raw_data, 0},
 {"", "get-all-raw-data", "",      N_("Get all raw data from folder"),   get_all_raw_data, 0},
 {"", "get-audio-data", "range",   N_("Get audio data given in range"),  get_audio_data, 0},
@@ -212,13 +219,14 @@ Option option[] = {
 {"", "list-config", "", N_("List the configuration tree"),    list_config, 0},
 {"", "get-config", "name", N_("Get a configuration variable"),    get_config, 0},
 {"", "set-config", "name", N_("Set a configuration variable"),    set_config, 0},
+{"", "wait-event",    "",  N_("Wait for event from camera"),  wait_event,   0},
 {"F" , "frames", "count", N_("Set number of frames to capture (default=infinite)"), capture_frames, 0},
 {"I" , "interval", "seconds", N_("Set capture interval in seconds"), capture_interval, 0},
 {"" , "capture-preview", "", N_("Capture a quick preview"), capture_preview, 0},
 {"" , "capture-image",  "",  N_("Capture an image"),        capture_image,   0},
 {"" , "capture-movie",  "",  N_("Capture a movie "),        capture_movie,   0},
 {"" , "capture-sound",  "",  N_("Capture an audio clip"),   capture_sound,   0},
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 {"", "show-exif", "range",   N_("Show EXIF information"), show_exif, 0},
 #endif
 {"", "show-info", "range",   N_("Show info"), show_info, 0},
@@ -482,7 +490,7 @@ OPTION_CALLBACK(show_info)
 	return (for_each_file_in_range (&p, print_info_action, arg));
 }
 
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 OPTION_CALLBACK(show_exif)
 {
 	/*
@@ -506,6 +514,11 @@ OPTION_CALLBACK (num_files)
 	return (num_files_action (&p));
 }
 
+OPTION_CALLBACK (wait_event)
+{
+	return (action_camera_wait_event (&p));
+}
+
 #endif
 
 static struct {
@@ -516,6 +529,7 @@ static struct {
 	{GP_FILE_TYPE_PREVIEW, "thumb_"},
 	{GP_FILE_TYPE_RAW, "raw_"},
 	{GP_FILE_TYPE_AUDIO, "audio_"},
+	{GP_FILE_TYPE_METADATA, "meta_"},
 	{0, NULL}
 };
 
@@ -569,7 +583,53 @@ get_path_for_file (const char *folder, CameraFile *file, char **path)
 	b[sizeof (b) - 1] = '\0';
 	for (i = 0; i < strlen (p.filename); i++) {
 		if (p.filename[i] == '%') {
-			switch (p.filename[++i]) {
+			char padding = '0'; /* default padding character */
+			int precision = 0;  /* default: no padding */
+			i++;
+			/* determine padding character */
+			switch (p.filename[i]) {
+			  /* case ' ':
+			   * spaces are not supported everywhere, so we
+			   * restrict ourselves to padding with zeros.
+			   */
+			case '0':
+				padding = p.filename[i];
+				precision = 1; /* do padding */
+				i++;
+				break;
+			}
+			/* determine padding width */
+			if (isdigit((int)p.filename[i])) {
+				char *cp;
+				long int _prec;
+				_prec = strtol(&p.filename[i],
+					       &cp, 10);
+				if (_prec < 1) 
+					precision = 1;
+				else if (_prec > 20)
+					precision = 20;
+				else
+					precision = _prec;
+				if (*cp != 'n') {
+					/* make sure this is %n */
+					gp_context_error (p.context,
+						  _("Zero padding numbers "
+						    "in file names is only "
+						    "possible with %%n."));
+					return GP_ERROR_BAD_PARAMETERS;
+				}
+				/* go to first non-digit character */
+				i += (cp - &p.filename[i]);
+			} else if (precision && ( p.filename[i] != 'n')) {
+				gp_context_error (p.context,
+					  _("You cannot use %%n "
+					    "zero padding "
+					    "without a "
+					    "precision value!"
+					    ));
+				return GP_ERROR_BAD_PARAMETERS;
+			}
+			switch (p.filename[i]) {
 			case 'n':
 
 				/*
@@ -590,11 +650,19 @@ get_path_for_file (const char *folder, CameraFile *file, char **path)
 					*path = NULL;
 					return (n);
 				}
-				snprintf (b, sizeof (b), "%i", n + 1);
+				if (precision > 1) {
+					char padfmt[16];
+					strcpy(padfmt, "%!.*i");
+					padfmt[1] = padding;
+					snprintf (b, sizeof (b), padfmt,
+						  precision, n + 1);
+				} else {
+					snprintf (b, sizeof (b), "%i",
+						  n + 1);
+				}
 				break;
 
 			case 'C':
-
 				/* Get the suffix of the original name */
 				s = strrchr (name, '.');
 				if (!s) {
@@ -696,13 +764,14 @@ save_camera_file_to_file (const char *folder, CameraFile *file)
 
         if ((p.flags & FLAGS_QUIET) == 0) {
                 while ((p.flags & FLAGS_FORCE_OVERWRITE) == 0 &&
-		       GP_SYSTEM_IS_FILE (s)) {
+		       gp_system_is_file (s)) {
 			do {
 				putchar ('\007');
 				printf (_("File %s exists. Overwrite? [y|n] "),
 					s);
 				fflush (stdout);
-				fgets (c, sizeof (c) - 1, stdin);
+				if (NULL == fgets (c, sizeof (c) - 1, stdin))
+					return GP_ERROR;
 			} while ((c[0]!='y')&&(c[0]!='Y')&&
 				 (c[0]!='n')&&(c[0]!='N'));
 
@@ -712,7 +781,8 @@ save_camera_file_to_file (const char *folder, CameraFile *file)
 			do { 
 				printf (_("Specify new filename? [y|n] "));
 				fflush (stdout); 
-				fgets (c, sizeof (c) - 1, stdin);
+				if (NULL == fgets (c, sizeof (c) - 1, stdin))
+					return GP_ERROR;
 			} while ((c[0]!='y')&&(c[0]!='Y')&&
 				 (c[0]!='n')&&(c[0]!='N'));
 
@@ -721,7 +791,8 @@ save_camera_file_to_file (const char *folder, CameraFile *file)
 
 			printf (_("Enter new filename: "));
 			fflush (stdout);
-			fgets (s, sizeof (s) - 1, stdin);
+			if (NULL == fgets (s, sizeof (s) - 1, stdin))
+				return GP_ERROR;
 			s[strlen (s) - 1] = 0;
                 }
                 printf (_("Saving file as %s\n"), s);
@@ -740,6 +811,8 @@ camera_file_exists (Camera *camera, GPContext *context, const char *folder,
 	CR (gp_camera_file_get_info (camera, folder, filename, &info,
 				     context));
 	switch (type) {
+	case GP_FILE_TYPE_METADATA:
+		return TRUE;
 	case GP_FILE_TYPE_AUDIO:
 		return (info.audio.fields != 0);
 	case GP_FILE_TYPE_PREVIEW:
@@ -823,6 +896,7 @@ get_file_common (const char *arg, CameraFileType type )
 {
         gp_log (GP_LOG_DEBUG, "main", "Getting '%s'...", arg);
 
+	p.download_type = type; /* remember for multi download */
 	/*
 	 * If the user specified the file directly (and not a number),
 	 * get that file.
@@ -847,6 +921,9 @@ get_file_common (const char *arg, CameraFileType type )
 	case GP_FILE_TYPE_EXIF:
 		CR (for_each_file_in_range (&p, save_exif_action, arg));
 		break;
+	case GP_FILE_TYPE_METADATA:
+		CR (for_each_file_in_range (&p, save_meta_action, arg));
+		break;
         default:
                 return (GP_ERROR_NOT_SUPPORTED);
         }
@@ -872,6 +949,11 @@ OPTION_CALLBACK (get_thumbnail)
         return (get_file_common (arg, GP_FILE_TYPE_PREVIEW));
 }
 
+OPTION_CALLBACK (get_metadata)
+{
+        return (get_file_common (arg, GP_FILE_TYPE_METADATA));
+}
+
 OPTION_CALLBACK(get_all_thumbnails)
 {
         CR (for_each_file (&p, save_thumbnail_action));
@@ -879,6 +961,13 @@ OPTION_CALLBACK(get_all_thumbnails)
 	return (GP_OK);
 }
 
+OPTION_CALLBACK (get_all_metadata)
+{
+        CR (for_each_file (&p, save_metadata_action));
+	return (GP_OK);
+}
+
+
 OPTION_CALLBACK (get_raw_data)
 {
         return (get_file_common (arg, GP_FILE_TYPE_RAW));
@@ -905,6 +994,7 @@ OPTION_CALLBACK (get_all_audio_data)
 
 OPTION_CALLBACK (delete_file)
 {
+	p.multi_type = MULTI_DELETE;
 	if (strchr (arg, '.'))
 		return (delete_file_action (&p, arg));
 
@@ -920,10 +1010,17 @@ OPTION_CALLBACK (delete_all_files)
 
 OPTION_CALLBACK (upload_file)
 {
-        gp_log (GP_LOG_DEBUG, "main", "Uploading file...");
-
+        gp_log (GP_LOG_DEBUG, "main", "Uploading file %s ...", arg);
+	p.multi_type = MULTI_UPLOAD;
 	CR (action_camera_upload_file (&p, p.folder, arg));
+        return (GP_OK);
+}
 
+OPTION_CALLBACK (upload_metadata)
+{
+        gp_log (GP_LOG_DEBUG, "main", "Uploading file %s ...", arg);
+	p.multi_type = MULTI_UPLOAD_META;
+	CR (action_camera_upload_metadata (&p, p.folder, arg));
         return (GP_OK);
 }
 
@@ -944,13 +1041,16 @@ OPTION_CALLBACK (remove_dir)
 }
 #endif
 
-static int
+int
 capture_generic (CameraCaptureType type, const char *name)
 {
 	CameraFilePath path, last;
 	char *pathsep;
 	int result, frames = 0;
+	time_t next_pic_time;
+	int waittime;
 
+	next_pic_time = time (NULL) + glob_interval;
 	if(glob_interval) {
 		memset(&last, 0, sizeof(last));
 		if (!(p.flags & FLAGS_QUIET))
@@ -1017,7 +1117,14 @@ capture_generic (CameraCaptureType type, const char *name)
 		 * a San Disk Ultra II CF).  May need to be increased for RAW
 		 * images...  I don't think this is the correct way to do this.
 		 */
+
+		/* Marcus Meissner: Fix the camera driver to only return from
+		 * camera_capture() when you know it is done ...
+		 * Otherwise other drivers suffer.
+		 */
+#if 0
 		sleep(2);
+#endif
 
 		result = get_file_common (path.name, GP_FILE_TYPE_NORMAL);
 		if (result != GP_OK) {
@@ -1048,6 +1155,12 @@ capture_generic (CameraCaptureType type, const char *name)
 		 */
 		if(glob_frames && frames == glob_frames) break;
 
+#if 0
+		/* Marcus Meissner: Before you enable this, try to fix the
+		 * camera driver first! camera_exit is NOT necessary for
+		 * 2 captures in a row!
+		 */
+
 		/* Without this, it seems that the second capture always fails.
 		 * That is probably obvious...  for me it was trial n' error.
 		 */
@@ -1055,11 +1168,17 @@ capture_generic (CameraCaptureType type, const char *name)
 		if (result != GP_OK) {
 			cli_error_print (_("Could not close camera connection."));
 		}
-
-		if (!(p.flags & FLAGS_QUIET) && glob_interval)
-			printf (_("Sleeping for %d second(s)...\n"), glob_interval);
-	
-		if(sleep(glob_interval) != 0) break;
+#endif
+		waittime = next_pic_time - time (NULL);
+		if (waittime > 0) {
+			if (!(p.flags & FLAGS_QUIET) && glob_interval)
+				printf (_("Sleeping for %d second(s)...\n"), glob_interval);
+			sleep (waittime);
+		} else {
+			if (!(p.flags & FLAGS_QUIET) && glob_interval)
+				printf (_("not sleeping (%d seconds behind schedule)\n"), - waittime);
+		}
+		next_pic_time += glob_interval;
 	}
 
 	return (GP_OK);
@@ -1255,15 +1374,18 @@ typedef enum {
 	ARG_FORCE_OVERWRITE,
 	ARG_GET_ALL_AUDIO_DATA,
 	ARG_GET_ALL_FILES,
+	ARG_GET_ALL_METADATA,
 	ARG_GET_ALL_RAW_DATA,
 	ARG_GET_ALL_THUMBNAILS,
 	ARG_GET_AUDIO_DATA,
 	ARG_GET_CONFIG,
 	ARG_SET_CONFIG,
 	ARG_GET_FILE,
+	ARG_GET_METADATA,
 	ARG_GET_RAW_DATA,
 	ARG_GET_THUMBNAIL,
 	ARG_LIST_CAMERAS,
+	ARG_LIST_CONFIG,
 	ARG_LIST_FILES,
 	ARG_LIST_FOLDERS,
 	ARG_LIST_PORTS,
@@ -1285,9 +1407,10 @@ typedef enum {
 	ARG_STDOUT_SIZE,
 	ARG_SUMMARY,
 	ARG_UPLOAD_FILE,
+	ARG_UPLOAD_METADATA,
 	ARG_USBID,
 	ARG_VERSION,
-	ARG_LIST_CONFIG
+	ARG_WAIT_EVENT,
 } Arg;
 
 typedef enum {
@@ -1452,7 +1575,7 @@ cb_arg (poptContext ctx, enum poptCallbackReason reason,
 		params->p.r = for_each_folder (&p, delete_all_action);
 		break;
 	case ARG_DELETE_FILE:
-
+		p.multi_type = MULTI_DELETE;
 		/* Did the user specify a file or a range? */
 		if (strchr (arg, '.')) {
 			params->p.r = delete_file_action (&p, arg);
@@ -1467,6 +1590,9 @@ cb_arg (poptContext ctx, enum poptCallbackReason reason,
 	case ARG_GET_ALL_FILES:
 		params->p.r = for_each_file (&p, save_file_action);
 		break;
+	case ARG_GET_ALL_METADATA:
+		params->p.r = for_each_file (&p, save_meta_action);
+		break;
 	case ARG_GET_ALL_RAW_DATA:
 		params->p.r = for_each_file (&p, save_raw_action);
 		break;
@@ -1474,15 +1600,23 @@ cb_arg (poptContext ctx, enum poptCallbackReason reason,
 		params->p.r = for_each_file (&p, save_thumbnail_action);
 		break;
 	case ARG_GET_AUDIO_DATA:
+		p.multi_type = MULTI_DOWNLOAD;
 		params->p.r = get_file_common (arg, GP_FILE_TYPE_AUDIO);
 		break;
+	case ARG_GET_METADATA:
+		p.multi_type = MULTI_DOWNLOAD;
+		params->p.r = get_file_common (arg, GP_FILE_TYPE_METADATA);
+		break;
 	case ARG_GET_FILE:
+		p.multi_type = MULTI_DOWNLOAD;
 		params->p.r = get_file_common (arg, GP_FILE_TYPE_NORMAL);
 		break;
 	case ARG_GET_THUMBNAIL:
+		p.multi_type = MULTI_DOWNLOAD;
 		params->p.r = get_file_common (arg, GP_FILE_TYPE_PREVIEW);
 		break;
 	case ARG_GET_RAW_DATA:
+		p.multi_type = MULTI_DOWNLOAD;
 		params->p.r = get_file_common (arg, GP_FILE_TYPE_RAW);
 		break;
 	case ARG_LIST_CAMERAS:
@@ -1541,8 +1675,13 @@ cb_arg (poptContext ctx, enum poptCallbackReason reason,
 		params->p.r = action_camera_summary (&p);
 		break;
 	case ARG_UPLOAD_FILE:
+		p.multi_type = MULTI_UPLOAD;
 		params->p.r = action_camera_upload_file (&p, p.folder, arg);
 		break;
+	case ARG_UPLOAD_METADATA:
+		p.multi_type = MULTI_UPLOAD_META;
+		params->p.r = action_camera_upload_metadata (&p, p.folder, arg);
+		break;
 	case ARG_LIST_CONFIG:
 		params->p.r = list_config_action (&p);
 		break;
@@ -1564,6 +1703,9 @@ cb_arg (poptContext ctx, enum poptCallbackReason reason,
 		free (name);
 		break;
 	}
+	case ARG_WAIT_EVENT:
+		params->p.r = action_camera_wait_event (&p);
+		break;
 	default:
 		break;
 	};
@@ -1691,6 +1833,12 @@ main (int argc, char **argv)
 		{"get-all-thumbnails", 'T', POPT_ARG_NONE, 0,
 		 ARG_GET_ALL_THUMBNAILS,
 		 N_("Get all thumbnails from folder"), NULL},
+		{"get-metadata", '\0', POPT_ARG_STRING, NULL, ARG_GET_METADATA,
+		 N_("Get metadata given in range"), NULL},
+		{"get-all-metadata", '\0', POPT_ARG_STRING, NULL, ARG_GET_ALL_METADATA,
+		 N_("Get all metadata from folder"), NULL},
+		{"upload-metadata", '\0', POPT_ARG_STRING, NULL, ARG_UPLOAD_METADATA,
+		 N_("Upload metadata for file"), NULL},
 		{"get-raw-data", '\0', POPT_ARG_STRING, NULL,
 		 ARG_GET_RAW_DATA,
 		 N_("Get raw data given in range"), NULL},
@@ -1719,6 +1867,8 @@ main (int argc, char **argv)
 		 N_("Get configuration value"), NULL},
 		{"set-config", '\0', POPT_ARG_STRING, NULL, ARG_SET_CONFIG,
 		 N_("Set configuration value"), NULL},
+		{"wait-event", '\0', POPT_ARG_NONE, NULL, ARG_WAIT_EVENT,
+		 N_("Wait for event from camera"), NULL},
 		{"capture-preview", '\0', POPT_ARG_NONE, NULL,
 		 ARG_CAPTURE_PREVIEW,
 		 N_("Capture a quick preview"), NULL},
@@ -1732,7 +1882,7 @@ main (int argc, char **argv)
 		 ARG_CAPTURE_MOVIE, N_("Capture a movie"), NULL},
 		{"capture-sound", '\0', POPT_ARG_NONE, NULL,
 		 ARG_CAPTURE_SOUND, N_("Capture an audio clip"), NULL},
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 		{"show-exif", '\0', POPT_ARG_STRING, NULL, ARG_SHOW_EXIF,
 		 N_("Show EXIF information"), NULL},
 #endif
@@ -1755,7 +1905,7 @@ main (int argc, char **argv)
 
 	/* For translation */
 	setlocale (LC_ALL, "");
-        bindtextdomain (PACKAGE, GPHOTO2_LOCALEDIR);
+        bindtextdomain (PACKAGE, LOCALEDIR);
         textdomain (PACKAGE);
 
 	/* Create the global variables. */
@@ -1959,6 +2109,44 @@ main (int argc, char **argv)
 	poptResetContext (ctx);
 	params.p.r = GP_OK;
 	while ((params.p.r >= GP_OK) && (poptGetNextOpt (ctx) >= 0));
+
+	switch (p.multi_type) {
+	case MULTI_UPLOAD: {
+		const char *arg;
+
+		while ((params.p.r >= GP_OK) && (NULL != (arg = poptGetArg (ctx)))) {
+			CR_MAIN (action_camera_upload_file (&p, p.folder, arg));
+		}
+		break;
+	}
+	case MULTI_UPLOAD_META: {
+		const char *arg;
+
+		while ((params.p.r >= GP_OK) && (NULL != (arg = poptGetArg (ctx)))) {
+			CR_MAIN (action_camera_upload_metadata (&p, p.folder, arg));
+		}
+		break;
+	}
+	case MULTI_DELETE: {
+		const char *arg;
+
+		while ((params.p.r >= GP_OK) && (NULL != (arg = poptGetArg (ctx)))) {
+			CR_MAIN (delete_file_action (&p, arg));
+		}
+		break;
+	}
+	case MULTI_DOWNLOAD: {
+		const char *arg;
+
+		while ((params.p.r >= GP_OK) && (NULL != (arg = poptGetArg (ctx)))) {
+			CR_MAIN (get_file_common (arg, p.download_type ));
+		}
+		break;
+	}
+	default:
+		break;
+	}
+
 	CR_MAIN (params.p.r);
 #else
         /* Count the number of command-line options we have */
