@@ -30,6 +30,10 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+
 #ifdef HAVE_RL
 #  include <readline/readline.h>
 #  include <readline/history.h>
@@ -68,6 +72,7 @@ static int shell_cd            (Camera *, const char *);
 static int shell_lcd           (Camera *, const char *);
 static int shell_exit          (Camera *, const char *);
 static int shell_get           (Camera *, const char *);
+static int shell_put           (Camera *, const char *);
 static int shell_get_thumbnail (Camera *, const char *);
 static int shell_get_raw       (Camera *, const char *);
 static int shell_del           (Camera *, const char *);
@@ -75,9 +80,14 @@ static int shell_help          (Camera *, const char *);
 static int shell_ls            (Camera *, const char *);
 static int shell_exit          (Camera *, const char *);
 static int shell_show_info     (Camera *, const char *);
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 static int shell_show_exif     (Camera *, const char *);
 #endif
+static int shell_list_config   (Camera *, const char *);
+static int shell_get_config    (Camera *, const char *);
+static int shell_set_config    (Camera *, const char *);
+static int shell_capture_image (Camera *, const char *);
+static int shell_mkdir         (Camera *, const char *);
 
 #define MAX_FOLDER_LEN 1024
 #define MAX_FILE_LEN 1024
@@ -100,8 +110,8 @@ struct _ShellFunctionTable {
 	{"lcd", shell_lcd, N_("Change to a directory on the local drive"),
 	 N_("directory"), 1},
 	{"exit", shell_exit, N_("Exit the gPhoto shell"), NULL, 0},
-	{"get", shell_get, N_("Download a file"), N_("[directory/]filename"),
-	 1},
+	{"get", shell_get, N_("Download a file"), N_("[directory/]filename"), 1},
+	{"put", shell_put, N_("Upload a file"), N_("[directory/]filename"), 1},
 	{"get-thumbnail", shell_get_thumbnail, N_("Download a thumbnail"),
 	 N_("[directory/]filename"), 1},
 	{"get-raw", shell_get_raw, N_("Download raw data"),
@@ -109,7 +119,8 @@ struct _ShellFunctionTable {
 	{"show-info", shell_show_info, N_("Show info"),
 	 N_("[directory/]filename"), 1},
 	{"delete", shell_del, N_("Delete"), N_("[directory/]filename"), 1},
-#ifdef HAVE_EXIF
+	{"mkdir", shell_mkdir, N_("Create Directory"), N_("directory"), 1},
+#ifdef HAVE_LIBEXIF
 	{"show-exif", shell_show_exif, N_("Show EXIF information"),
 	 N_("[directory/]filename"), 1},
 #endif
@@ -117,6 +128,10 @@ struct _ShellFunctionTable {
 	 N_("[command]"), 0},
 	{"ls", shell_ls, N_("List the contents of the current directory"),
 	 N_("[directory/]"), 0},
+	{"list-config", shell_list_config, N_("List configuration variables"), NULL, 0},
+	{"get-config", shell_get_config, N_("Get configuration variable"), N_("name"), 1},
+	{"set-config", shell_set_config, N_("Set configuration variable"), N_("name=value"), 1},
+	{"capture-image", shell_capture_image, N_("Capture a single image"), N_("[name]"), 0},
 	{"q", shell_exit, N_("Exit the gPhoto shell"), NULL, 0},
 	{"quit", shell_exit, N_("Exit the gPhoto shell"), NULL, 0},
 	{"?", shell_help, N_("Displays command usage"), N_("[command]"), 0},
@@ -133,11 +148,11 @@ shell_arg_count (const char *args)
 	int x=0, in_arg=0, count=0;
 	
 	while (x < strlen (args)) {
-		if ((!isspace(args[x])) && (!in_arg)) {
+		if ((!isspace((int)(args[x]))) && (!in_arg)) {
 			in_arg = 1;
 			count++;
 		}
-		if ((isspace(args[x])) && (in_arg))
+		if ((isspace((int)(args[x]))) && (in_arg))
 			in_arg = 0;
 		x++;
 	}
@@ -148,7 +163,10 @@ shell_arg_count (const char *args)
 static char *
 shell_read_line (void)
 {
-	char prompt[70], buf[1024], *line, *tmp;
+	char prompt[70], buf[1024], *line;
+#ifndef HAVE_RL
+	char *tmp;
+#endif
 
 	if (p->flags & FLAGS_QUIET)
 		snprintf (prompt, sizeof (prompt), SHELL_PROMPT, "\0", "\0");
@@ -192,13 +210,13 @@ shell_arg (const char *args, int arg_num, char *arg)
 		return (GP_ERROR);
 
 	while (x < strlen(args)) {				/* Edge-triggered */
-		if ((!isspace(args[x])) && (!in_arg)) {
+		if ((!isspace((int)(args[x]))) && (!in_arg)) {
 			in_arg = 1;
 			if (count == arg_num)
 				copy = 1;
 			count++;
 		}
-		if ((isspace(args[x])) && (in_arg)) {
+		if ((isspace((int)(args[x]))) && (in_arg)) {
 			copy = 0;
 			in_arg = 0;
 		}
@@ -404,8 +422,20 @@ shell_prompt (GPParams *params)
 
 	while (!shell_done && !glob_cancel) {
 		line = shell_read_line ();
-		if (!line)
-			continue;
+		if (line ==  NULL) {
+			/* quit shell on EOF or input error */
+			printf("\n");
+			fflush(stdout);
+			break;
+		}
+#ifdef HAVE_UNISTD_H
+		if (!isatty(fileno(stdin))) {
+			/* if non-interactive input, the command has not been
+			 * printed yet, so we do that here */
+			printf("%s\n", line);
+			fflush(stdout);
+		}
+#endif
 
 		/* If we don't have any command, start from the beginning */
 		if (shell_arg_count (line) <= 0) {
@@ -695,7 +725,7 @@ shell_del (Camera *camera, const char *arg)
 	return (GP_OK);
 }
 
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 static int
 shell_show_exif (Camera *camera, const char *arg)
 {
@@ -715,6 +745,73 @@ shell_show_info (Camera *camera, const char *arg)
 	return (GP_OK);
 }
 
+static int
+shell_put (Camera *camera, const char *args) {
+	char arg[1024];
+	unsigned int x;
+	char dest_folder[MAX_FOLDER_LEN], dest_filename[MAX_FILE_LEN];
+
+	for (x = 0; x < shell_arg_count (args); x++) {
+		CHECK (shell_arg (args, x, arg));
+		CHECK (shell_construct_path ("/", arg, dest_folder, dest_filename));
+		CHECK (action_camera_upload_file (p, dest_folder, dest_filename));
+	}
+	
+	return (GP_OK);      
+}
+
+static int
+shell_mkdir (Camera *camera, const char *args) {
+	if (*args == ' ')
+		args++;
+	return gp_camera_folder_make_dir (camera, p->folder, args, p->context);
+}
+
+static int
+shell_list_config (Camera *camera, const char *args) {
+	CHECK (list_config_action (p));
+	return (GP_OK);      
+}
+
+static int
+shell_get_config (Camera *camera, const char *args) {
+	char arg[1024];
+	unsigned int x;
+
+	for (x = 0; x < shell_arg_count (args); x++) {
+		CHECK (shell_arg (args, x, arg));
+		CHECK (get_config_action (p, arg));
+	}
+	return (GP_OK);      
+}
+
+static int
+shell_set_config (Camera *camera, const char *args) {
+	char arg[1024];
+	char *s,*x;
+
+	strncpy (arg, args, sizeof(arg));
+	arg[1023]='\0';
+	/* need to skip spaces */
+	x = arg; while (*x == ' ') x++;
+	if ((s=strchr(x,'='))) {
+		*s='\0';
+		return set_config_action (p, x, s+1);
+	}
+	if ((s=strchr(x,' '))) {
+		*s='\0';
+		return set_config_action (p, x, s+1);
+	}
+	fprintf (stderr, _("set-config needs a second argument.\n"));
+	return (GP_OK);      
+}
+
+static int
+shell_capture_image (Camera *camera, const char *args) {
+	while (*args==' ') args++;
+	return capture_generic (GP_CAPTURE_IMAGE, *args?args:NULL);
+}
+
 int
 shell_exit (Camera *camera, const char *arg)
 {
